% This is the main script for experimental results of the manuscript titled
% "INTELLIGIBILITY ENHANCEMENT BASED ON MUTUAL INFORMATION"
% Seyran Khademi
% R. C. Hendriks
% W. B. Kleijn
% June 2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
clc;
addpath(genpath(fullfile(cd,filesep,'RIR_Generator_v2.0',filesep)));
addpath(genpath(fullfile(cd,filesep,'General_Func',filesep)));
%%
%%SET THE PARAMETERS%
fs=16000;                       %set the sampling frequency
fft_size=round(fs*32/1000);                   %set the fft size (frame size)
Lr = [3 4 3];                   % Room dimensions [x y z] (m)
Nm=3;                           % set the number of microphones
rp = [[1.5 2 1];[1.5 2 1]+[0 0.02,0];[1 2 2.2]];    % set the mic. position
sp = [[1.5 2 1]+[0 1 0];[1.5 2 1]+[-1 -1 0];...
    [1.5 2 1]+[-0.75 +1 0]; [1.5 2 1]+[+1.5 -0.8 0]]; % set the noise source positions
nearend_SNR=0; % set the nearend SNRs that are evaluated 
farend_SNR=-10;  % set the farend SNRs that are evaluated 
%% reading the clean speach signal

[s,FS]=audioread('./audiodata/clean.wav'); % clean signal
s =resample(s,fs,FS);
s=[zeros(fs/2,1);s];

%% producing the channel impulse responses

Nu=length(sp(:,1))-1; % number of noise sources
for nu=1:Nu+1
    d(:,:,nu)=generate_impulse_resp(rp,sp(nu,:),fs,Lr,0);%
    norm1=max(d(1,:,nu));
    norm2=max(d(2,:,nu));
    d(1,:,nu)=d(1,:,nu)/norm1; 
    d(2,:,nu)=d(2,:,nu)/norm2;
end
%% 
%generating the near-end (n) and far-end (u) noise (for Nu noise sources at far-end)
% generate noise at far-end generate a synthetic noise source: To create an
% extreme situation, chose spectra from n and u non-overlapping.
[u,n]=generate_NS(Nu);

while(length(u)<length(s))
    u=[u;u];
end
u=u(1:length(s),:);
while(length(n)<length(s))
    n=[n;n];
end
n=n(1:length(s),:);

%%%Calculating the ATFs for all sources.

for l=1:Nm
    for nu=1:Nu+1
        D_full     = fft(d(l,:,nu),fft_size); 	% Transform impulse response to DFT domain
        D(l,:,nu) =D_full(1:fft_size/2+1);
    end
end
%% scaling to set the SNR
len=length(s);
mic_noise=60; %dB
for I=1:length(nearend_SNR)
    for II=[1:length(farend_SNR)]%1:length(SNRmat2)
        SNR.nearend=nearend_SNR(I)
        SNR.farend=farend_SNR(II)
        SPL=62.35; %dB
        scale=sqrt(mean(s.^2)); % scale to put original at 120 dB SPL
        s=s*(10^((SPL-120)/20))/scale;
        snr1 = 10*log10(var(s)./diag(var(n)))   ;
        dif1 = snr1-nearend_SNR(I);
        n    = n .* 10^(dif1/20); % scale the near-end noise
        % Mic self noise
        un_corr=randn(length(n),Nm);
        for l=1:Nm
            snr2(l) = 10*log10(var(s)./diag(var(un_corr(:,l)))) ;
            dif2(l)= snr2(l)-(mic_noise);
            un_corr(:,l) = un_corr(:,l) .* 10.^(dif2(l)./20); % scale the noise
        end
        % scale noise realizations of noise sources to right SNRs
        for l=1:Nu
            snr2 = 10*log10(var(s)./diag(var(u(:,l)))) ;
            dif2= snr2-(farend_SNR(II));
            u(:,l) = u (:,l).* 10.^(dif2./20); % scale the far-noise
        end
       

        %% finding the optimal processor
        [MVDR_out_post_with_near_end,noisy_unprocessed,mvdr_out_with_near_end] = generate_data_and_mvdr(s, n,u,un_corr,fs,D,fft_size);
   
       
    end
end
%%

%%